/*  CommonFnc.cpp -- generaly usable function library  
 
    This is part of LightApp, demostration software agent using SecureFW protective interface.
    See http://www.svenda.com/petr/dpthesis/index.html for more information
    Copyright (C) 2004  Petr Svenda <petr@svenda.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please, report any bugs to author <petr@svenda.com>
/**/

#include "stdafx.h"
#include "CommonFnc.h"

int CCommonFnc::BYTE_ConvertFromHexStringToArray(CString hexaString, BYTE* pArray, DWORD* pbArrayLen) {
    int         status = STAT_OK;
    DWORD       pos = 0;
    DWORD       pos2 = 0;
    CString     hexNum;
    DWORD       num;
    BYTE*       pTempArray = NULL;
    DWORD       tempArrayPos = 0;

    // EAT SPACES
    hexaString.Replace(" ", "");
    hexaString.TrimLeft(); hexaString.TrimRight();
    // SEPARATE HEX VALUES BY SPACE
    status = String_SeparateHexValues(&hexaString, " ");
    hexaString += " ";
    hexaString.GetLength();

    if (status == STAT_OK) {
        pTempArray = new BYTE[hexaString.GetLength()];
        memset(pTempArray, 0, hexaString.GetLength());

        pos = pos2 = 0;
        while ((pos = hexaString.Find(' ', pos2)) != -1) {
            hexNum = hexaString.Mid(pos2, pos - pos2);
            hexNum.TrimLeft(); hexNum.TrimRight();
            if (hexNum.GetLength() > 0) {
                num = strtol((LPCTSTR) hexNum, NULL, 16);
        
                pTempArray[tempArrayPos] = (BYTE) num & 0xFF;
                tempArrayPos++;
            }
            pos2 = pos + 1;
        }

        if (tempArrayPos > *pbArrayLen) {
            status = STAT_NOT_ENOUGHT_MEMORY;
        }  
        else {
            memcpy(pArray, pTempArray, tempArrayPos);
        }
        *pbArrayLen = tempArrayPos;

        if (pTempArray) delete[] pTempArray;
    }

    return status;
}

int CCommonFnc::BYTE_ConvertFromHexStringToArray(CString hexaString, BYTE* pArray, USHORT* pbArrayLen) {
    int     status = STAT_OK;
    DWORD   arrayLen = *pbArrayLen;    
    
    status = BYTE_ConvertFromHexStringToArray(hexaString, pArray, &arrayLen);
    if (arrayLen > 0xFFFF) status = STAT_NOT_ENOUGHT_DATA_TYPE;
    else *pbArrayLen = (USHORT) arrayLen;

    return status;
}

int CCommonFnc::BYTE_ConvertFromArrayToHexString(BYTE* pArray, DWORD pbArrayLen, CString* pHexaString) {
    int         status = STAT_OK;
    CString     hexNum;
    DWORD       i;

    *pHexaString = "";
    for (i = 0; i < pbArrayLen; i++) {
        hexNum.Format("%.2x", pArray[i]);
        hexNum += " ";

        *pHexaString += hexNum;
    }

    return status;
}

int CCommonFnc::BYTE_ConvertFromArrayToString(BYTE* pArray, DWORD pbArrayLen, CString* pString) {
    int     status = STAT_OK;
    CString hexString;

    if ((status = CCommonFnc::BYTE_ConvertFromArrayToHexString(pArray, pbArrayLen, &hexString)) == STAT_OK) {
        status = String_ConvertFromHexToString(hexString, pString);    
    } 
    
    return status;
}

int CCommonFnc::String_SeparateHexValues(CString* pHexString, CString separator) {
    int     status = STAT_OK;
    int     i;    
    CString temp = "";    

    pHexString->TrimLeft(); pHexString->TrimRight();

    // TRANSFORM FROM FORM LIKE '010E060C' INTO '01 0E 06 0C' 
    for (i = 0; i < pHexString->GetLength(); i++) {
        if (i % 2 == 0) temp += separator;                
        temp += pHexString->GetAt(i);                
    } 

    temp.TrimLeft(); temp.TrimRight();

    *pHexString = temp;
        
    return status;
}

int CCommonFnc::String_SeparateLines(CString* pString, int charsPerLine, CString separator) {
    int     status = STAT_OK;
    int     i = 0;
    int     origLen = pString->GetLength();
    int     offset = 0;

    while ((i + charsPerLine) < origLen) {
        offset += charsPerLine;
        pString->Insert(offset, separator);
        offset += strlen((LPCTSTR) separator);
        
        i += charsPerLine;   
    }

    return status;
}

int CCommonFnc::String_ConvertFromStringToHex(CString string, CString* pHexString) {
    int     status = STAT_OK;
    int     i;
    TCHAR   c;
    CString hexC;

    *pHexString = "";
    for (i = 0; i < string.GetLength(); i++) {
        c = string.GetAt(i);
        
        hexC.Format("%.2X", c);
        *pHexString += hexC;     
    }	

    status = Hex_ValidateHexString(pHexString);
      
    return status;
}

int CCommonFnc::String_ConvertFromHexToString(CString hexString, CString* pString) {
    int     status = STAT_OK;
    int     i;
    CString charStr;
    BYTE*   hexArray = NULL;
    USHORT  hexArrayLen;

    if (status == STAT_OK) {
        status = CCommonFnc::Hex_ValidateHexString(&hexString);
    }
    if (status == STAT_OK) {
        hexArrayLen = hexString.GetLength();
        hexArray = new BYTE[hexArrayLen];
        memset(hexArray, 0, hexArrayLen);
        status = CCommonFnc::BYTE_ConvertFromHexStringToArray(hexString, hexArray, &hexArrayLen);
    }
    if (status == STAT_OK) {
        *pString = "";
        for (i = 0; i < hexArrayLen; i++) {
            charStr.Format("%c", hexArray[i]);
        
            *pString += charStr;     
        }	
    }

    if (hexArray) delete[] hexArray;

    return status;
}

int CCommonFnc::Hex_ValidateHexString(CString* pHexString) {
    int     status = STAT_OK;
    
    // REMOVE SPACES
    pHexString->Replace(" ", "");
    pHexString->Replace("\n", "");
    pHexString->Replace("\r", "");
    
    // SEPARE HEX VALUES
    status = String_SeparateHexValues(pHexString, " ");

    // TODO: VERIFY CORRECT VALUES (0x00 - 0xFF)


    return status;
}

int CCommonFnc::Time_GetTimeSpan(SYSTEMTIME* pStartTime, SYSTEMTIME* pEndTime, FILETIME* pFileTimeSpan) {
    int             status = STAT_OK;
    FILETIME        startFileTime;
    FILETIME        endFileTime;

    memset(&startFileTime, 0, sizeof(FILETIME));
    memset(&endFileTime, 0, sizeof(FILETIME));

    SystemTimeToFileTime(pStartTime, &startFileTime);
    SystemTimeToFileTime(pEndTime, &endFileTime);

    *((__int64*) pFileTimeSpan) = (__int64) (*((__int64*) &endFileTime) - *((__int64*) &startFileTime));

    return status;
}

int CCommonFnc::Time_GetTimeSpanSec(SYSTEMTIME* pStartTime, SYSTEMTIME* pEndTime, float* pFloatTimeSpan) {
    int         status = STAT_OK;
    FILETIME    fileTimeSpan;

    memset(&fileTimeSpan, 0, sizeof(FILETIME));
 
    if ((status = Time_GetTimeSpan(pStartTime, pEndTime, &fileTimeSpan)) == STAT_OK) {
        *pFloatTimeSpan = *((__int64*) &fileTimeSpan) / (float) FILETIME_TO_SECOND;
    }

    return status;
}

