/*  TabAlgLicence.cpp -- Algorithm licence dialog service functions implementation   
 
    This is part of LightApp, demostration software agent using SecureFW protective interface.
    See http://www.svenda.com/petr/dpthesis/index.html for more information
    Copyright (C) 2004  Petr Svenda <petr@svenda.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please, report any bugs to author <petr@svenda.com>
/**/


#include "stdafx.h"
#include "LightApp.h"
#include "TabAlgLicence.h"
#include "LightAppDlg.h"
#include "CommonFnc.h"
#include "..\SecureFW\SecFunctProvider.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CLightAppDlg*  pLightApp;
/////////////////////////////////////////////////////////////////////////////
// CTabAlgLicence dialog


CTabAlgLicence::CTabAlgLicence(CWnd* pParent /*=NULL*/)
	: CDialog(CTabAlgLicence::IDD, pParent)
{
	//{{AFX_DATA_INIT(CTabAlgLicence)
	m_outOfOrderLicence = FALSE;
	m_algLicenceText = _T("");
	m_cipherBeforeSend = FALSE;
	m_licenceKey = _T("");
	m_aLicHexa = FALSE;
	m_noIOBufferSecurity = TRUE;
	//}}AFX_DATA_INIT
}


void CTabAlgLicence::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTabAlgLicence)
	DDX_Check(pDX, IDC_OUTOFORDER_LICENCE, m_outOfOrderLicence);
	DDX_Text(pDX, IDC_ALGLICENCETEXT, m_algLicenceText);
	DDX_Check(pDX, IDC_CIPHERBEFORESEND, m_cipherBeforeSend);
	DDX_Text(pDX, IDC_LICENCEKEY, m_licenceKey);
	DDX_Check(pDX, IDC_ALGLIC_HEXADECIMAL, m_aLicHexa);
	DDX_Check(pDX, IDC_ALIC_NOIOBUFFERSECURITY, m_noIOBufferSecurity);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CTabAlgLicence, CDialog)
	//{{AFX_MSG_MAP(CTabAlgLicence)
	ON_BN_CLICKED(IDC_LOADFROMFILE, OnLoadfromfile)
	ON_BN_CLICKED(IDC_PROCESSLICENCE, OnProcesslicence)
	ON_BN_CLICKED(IDC_ALGLIC_HEXADECIMAL, OnAlglicHexadecimal)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTabAlgLicence message handlers

void CTabAlgLicence::OnLoadfromfile() 
{
    int         status = STAT_OK;
    CFile       file;
    CFileDialog fileDialog(TRUE);

    UpdateData(TRUE);

    if (fileDialog.DoModal() == IDOK) {
        if (file.Open(fileDialog.GetPathName(), CFile::modeRead)) {
            // READ LICENCE FROM FILE
            char*   fileContent;
            fileContent = new char[file.GetLength() + 1];  
            memset(fileContent, 0, file.GetLength() + 1);
            file.Read(fileContent, file.GetLength());
            file.Close();   
            
            m_algLicenceText = fileContent;
            delete[] fileContent; 
        }
    }

    m_algLicenceText.Replace("><", ">\r\n<");
	UpdateData(FALSE);
}

void CTabAlgLicence::OnProcesslicence() 
{
    int             status = STAT_OK;
    ALG_LICENCE     algLicence;
    CString         licenceText;
    CString         message;
    CString         timeMessage;
    SYSTEMTIME      startTime;
    SYSTEMTIME      endTime;
    float           floatTimeSpan;
    BYTE            debugFlags = 0;
    DWORD           maxDataLen; 
    DWORD           dataLen; 
    BYTE            rawKey[SESSION_KEY_LEN]; 
    USHORT          rawKeyLength = SESSION_KEY_LEN;

    UpdateData(TRUE);

    //
    //  ALG LICENCE PREPARATION
    //
    if (status == STAT_OK) {
        m_algLicenceText.TrimLeft(); m_algLicenceText.TrimRight();

        // CONVERT INPUT VALUE INTO HEXADECIMAL FORMAT LIKE '01 05 0E ...'
        if (m_aLicHexa) {
            licenceText = m_algLicenceText;
            status = CCommonFnc::Hex_ValidateHexString(&licenceText);
        }
        else status = CCommonFnc::String_ConvertFromStringToHex(m_algLicenceText, &licenceText);
    }

    //
    //  INPUT DATA CHECKS
    //
    if (licenceText.IsEmpty()) {
        MessageBox(STR_ALGLICENCE_REQUIRED, STR_APP_NAME, MB_OK);
        CButton*    button;
        if ((button = (CButton*) GetDlgItem(IDC_ALGLICENCETEXT))) {
            button->SetFocus();
        }

        status = STAT_USERDATA_BAD;
    }
    if (m_cipherBeforeSend && m_licenceKey.IsEmpty()) {
        MessageBox(STR_LICENCE_KEY_REQUIRED, STR_APP_NAME, MB_OK);
        CButton*    button;
        if ((button = (CButton*) GetDlgItem(IDC_LICENCEKEY))) {
            button->SetFocus();
        }

        status = STAT_USERDATA_BAD;
    }

    // 
    //  DEBUG FLAGS 
    //
    if (m_noIOBufferSecurity) debugFlags |= FLAG_DEBUG_DISABLE_IOBUFFER_SECURING;


    if (status == STAT_OK) {
        licenceText.TrimLeft(); m_algLicenceText.TrimRight();

        algLicence.dataLen = LICENCE_MAX_LENGTH;
        algLicence.pData = new BYTE[algLicence.dataLen];
        memset(algLicence.pData, 0, algLicence.dataLen);
    
        if (m_cipherBeforeSend && !m_noIOBufferSecurity) {
            // PREPARE RAW KEY
            if ((status = CCommonFnc::Hex_ValidateHexString(&m_licenceKey)) == STAT_OK) {
                memset(rawKey, 0, SESSION_KEY_LEN);
                if ((status = CCommonFnc::BYTE_ConvertFromHexStringToArray(m_licenceKey, rawKey, &rawKeyLength)) == STAT_OK) {
                    // CHECK EXPECTED LENGTH == SESSION_KEY_LEN
                    if (rawKeyLength != SESSION_KEY_LEN) status = STAT_KEY_LENGTH_BAD;
                }
            }
            
            // ENCRYPT LICENCE
            if (status == STAT_OK) {
                if ((status = CCommonFnc::BYTE_ConvertFromHexStringToArray(licenceText, algLicence.pData, &algLicence.dataLen)) == STAT_OK) {
                    maxDataLen = LICENCE_MAX_LENGTH;
                    dataLen = algLicence.dataLen;
                    if ((status = CSecFunctProvider::AES128_EncryptCBC(algLicence.pData, &dataLen, &maxDataLen, rawKey, IV)) == STAT_OK) {
                        algLicence.dataLen = (USHORT) dataLen;    
                    }
                }
            }
        }    
        else {
            // ENCRYPTED BEFORE, ONLY CONVERT TO BYTE ARRAY
            CCommonFnc::BYTE_ConvertFromHexStringToArray(licenceText, algLicence.pData, &algLicence.dataLen);
        }
    }

    //  OUTPUT ERROR, IF OCCURED  
    if (status != STAT_OK) {
        message.Format("Process licence fail with %s (%d)", ERROR_TO_STRING(status), status);
        pLightApp->Log_InsertClearLine();
        pLightApp->Log_InsertString(message);
    }

    //
    // SEND LICENCE
    //
    if (status == STAT_OK) {
        memset(&startTime, 0, sizeof(SYSTEMTIME));
        memset(&endTime, 0, sizeof(SYSTEMTIME));

        pLightApp->DisplayActualOperation(STR_ACTIVEOPER_ALGLICENCE);

        GetSystemTime(&startTime); 
        // PROCESS LICENCE
        if ((status = pLightApp->m_remoteManager.ProcessAlgLicence(&algLicence, m_outOfOrderLicence, debugFlags)) == STAT_OK) {
            GetSystemTime(&endTime); 
            message = "Process licence success";
        }
        else {
            GetSystemTime(&endTime); 
            message.Format("Process licence fail with %s (%d)", ERROR_TO_STRING(status), status);
        }
        
        pLightApp->DisplayActualOperation();

        CCommonFnc::Time_GetTimeSpanSec(&startTime, &endTime, &floatTimeSpan);

        timeMessage.Format("--- elapsed timer (seconds): %f", floatTimeSpan);
        pLightApp->Log_InsertClearLine();
        pLightApp->Log_InsertString(timeMessage);
        pLightApp->Log_InsertString(message);
    }
  
    UpdateData(FALSE);
}

void CTabAlgLicence::OnAlglicHexadecimal() 
{
    CString     temp;
    UpdateData(TRUE);

    if (m_aLicHexa) {
        // CONVERT FROM STRING TO HEX
        CCommonFnc::String_ConvertFromStringToHex(m_algLicenceText, &temp);
        m_algLicenceText = temp;

        CCommonFnc::String_SeparateLines(&m_algLicenceText, 69);
    }
    else {
        // CONVERT FROM HEX TO STRING
        CCommonFnc::String_ConvertFromHexToString(m_algLicenceText, &temp);
        m_algLicenceText = temp;
    }

    UpdateData(FALSE);
}
