/*  TabSafeAlg.cpp -- interactive usage dialog of Protected algorithm   
 
    This is part of LightApp, demostration software agent using SecureFW protective interface.
    See http://www.svenda.com/petr/dpthesis/index.html for more information
    Copyright (C) 2004  Petr Svenda <petr@svenda.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please, report any bugs to author <petr@svenda.com>
/**/


#include "stdafx.h"
#include "LightApp.h"
#include "TabSafeAlg.h"
#include "CommonFnc.h"
#include "LightAppDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CLightAppDlg*  pLightApp;
/////////////////////////////////////////////////////////////////////////////
// CTabSafeAlg dialog


CTabSafeAlg::CTabSafeAlg(CWnd* pParent /*=NULL*/)
	: CDialog(CTabSafeAlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CTabSafeAlg)
	m_UUIDSWA = _T("");
	m_safeAlg = -1;
	m_uuidAlgCustom = _T("");
	m_uuidAlgTest1 = _T("");
	m_uuidAlgTest2 = _T("");
	m_inputData = _T("1998");
	m_outputData = _T("");
	m_inputDataHex = FALSE;
	m_outputDataHex = FALSE;
	//}}AFX_DATA_INIT
#ifdef _DEBUG    
    m_seautNoIOBufferSecurity = TRUE;
	m_safeAlgNoIOBufferSecurity = TRUE;
#else 
    m_seautNoIOBufferSecurity = FALSE;
	m_safeAlgNoIOBufferSecurity = FALSE;
#endif
    m_UUIDSWA = UUIDSWA;
    m_uuidAlgTest1 = UUIDALG_GAUSS;
}


void CTabSafeAlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTabSafeAlg)
	DDX_Text(pDX, IDC_UUIDSWA, m_UUIDSWA);
	DDX_Radio(pDX, IDC_SAFEALG_1, m_safeAlg);
	DDX_Text(pDX, IDC_UUID_ALGCUSTOM, m_uuidAlgCustom);
	DDX_Text(pDX, IDC_UUID_ALGTEST1, m_uuidAlgTest1);
	DDX_Text(pDX, IDC_INPUTDATA, m_inputData);
	DDX_Text(pDX, IDC_OUTPUTDATA, m_outputData);
	DDX_Check(pDX, IDC_SEAUT_NOIOBUFFERSECURITY, m_seautNoIOBufferSecurity);
	DDX_Check(pDX, IDC_SEFEALG_NOIOBUFFERSECURITY, m_safeAlgNoIOBufferSecurity);
	DDX_Check(pDX, IDC_INPUTDATA_HEXADECIMAL, m_inputDataHex);
	DDX_Check(pDX, IDC_OUTPUTDATA_HEXADECIMAL, m_outputDataHex);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CTabSafeAlg, CDialog)
	//{{AFX_MSG_MAP(CTabSafeAlg)
	ON_BN_CLICKED(IDC_PROCESS_AUTH, OnProcessAuth)
	ON_BN_CLICKED(IDC_USESAFEALG, OnUsesafealg)
	ON_BN_CLICKED(IDC_STOP_AUTH, OnStopAuth)
	ON_EN_CHANGE(IDC_INPUTDATA, OnChangeInputdata)
	ON_BN_CLICKED(IDC_INPUTDATA_HEXADECIMAL, OnInputdataHexadecimal)
	ON_BN_CLICKED(IDC_OUTPUTDATA_HEXADECIMAL, OnOutputdataHexadecimal)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTabSafeAlg message handlers

void CTabSafeAlg::OnProcessAuth() 
{
    UpdateData(TRUE);

    int             status = STAT_OK;
    CString         message;
    CString         timeMessage;
    SWA_PROFILE     swaProfile;
    DWORD           length = 0;
    BYTE            debugFlags = 0;
    SYSTEMTIME      startTime;
    SYSTEMTIME      endTime;
    float           floatTimeSpan;

    
    UpdateData(TRUE);
    //TODO: Test correct values
    if (m_seautNoIOBufferSecurity) debugFlags |= FLAG_DEBUG_DISABLE_IOBUFFER_SECURING;

    // UUIDSWA
    length = SWA_UUID_LENGTH;
    CCommonFnc::BYTE_ConvertFromHexStringToArray(m_UUIDSWA, swaProfile.swaUUID, &length);
    swaProfile.swaUUIDLen = (BYTE) length;

    memset(&startTime, 0, sizeof(SYSTEMTIME));
    memset(&endTime, 0, sizeof(SYSTEMTIME));

    pLightApp->DisplayActualOperation(STR_ACTIVEOPER_SWAAUTH);

    // PROCESS AUTH BY SEAUT
    GetSystemTime(&startTime); 
    if ((status = pLightApp->m_remoteManager.ProcessAuth(&swaProfile, debugFlags)) == STAT_OK) {
        GetSystemTime(&endTime); 
        message = "ProcessAuth success";

        pLightApp->SetAuthSWA(TRUE, m_UUIDSWA);
    }
    else {
        GetSystemTime(&endTime); 
        message.Format("ProcessAuth fail with %s (%d)", ERROR_TO_STRING(status), status);
    }

    pLightApp->DisplayActualOperation();

    CCommonFnc::Time_GetTimeSpanSec(&startTime, &endTime, &floatTimeSpan);

    timeMessage.Format("--- elapsed timer (seconds): %f", floatTimeSpan);
    pLightApp->Log_InsertClearLine();
    pLightApp->Log_InsertString(timeMessage);
    pLightApp->Log_InsertString(message);


    UpdateData(FALSE);
}

void CTabSafeAlg::OnUsesafealg() 
{
    int                 status = STAT_OK;
    CString             algUUIDStr;
    CString             message;
    CString             timeMessage;
    CString             inputData;
    SYSTEMTIME          startTime;
    SYSTEMTIME          endTime;
    float               floatTimeSpan;
    USHORT              year = 0;
    
    UpdateData(TRUE);

    switch (m_safeAlg) {
        case 0: {
            algUUIDStr = m_uuidAlgTest1;
            
            // GAUSS EASTER ALGORITHM, EXPECTED VALUE IS YEAR NUMBER IN STRING ASCII FORMAT '1998'
            if (m_inputDataHex) status = CCommonFnc::String_ConvertFromHexToString(m_inputData, &inputData);
            else inputData = m_inputData;

            if (status == STAT_OK) {
                year = (USHORT) atol((LPCTSTR) inputData); 
           
                if ((year != 0) && (year > 1900) && (year < 2099))  {
                    BYTE    shortArray[2];
                    shortArray[0] = ((BYTE*) &year)[1];
                    shortArray[1] = ((BYTE*) &year)[0];
                    CCommonFnc::BYTE_ConvertFromArrayToHexString(shortArray, sizeof(USHORT), &inputData);
                    CCommonFnc::Hex_ValidateHexString(&inputData);
                }
                else {
                    MessageBox(STR_EASTER_YEAR_REQUIRED, STR_APP_NAME, MB_OK | MB_ICONWARNING);
                    status = STAT_DATA_INCORRECT_LENGTH;
                }
            }
          
            break;
        }
        case 1: {
            algUUIDStr = m_uuidAlgCustom;
            
            // CONVERT INPUT VALUE INTO HEXADECIMAL FORMAT LIKE '01 05 0E ...'
            if (m_inputDataHex) {
                status = CCommonFnc::Hex_ValidateHexString(&m_inputData);
                inputData = m_inputData;
            }
            else status = CCommonFnc::String_ConvertFromStringToHex(m_inputData, &inputData);
            
            break;
        }
    }

    //  OUTPUT ERROR, IF OCCURED  
    if (status != STAT_OK) {
        message.Format("Process licence fail with %s (%d)", ERROR_TO_STRING(status), status);
        pLightApp->Log_InsertClearLine();
        pLightApp->Log_InsertString(message);
    }

    if (status == STAT_OK) {
        DATA_BLOB   input;
        DATA_BLOB   output;
        DATA_BLOB   algUUID;
        BYTE        debugFlags = 0;
    
        if (m_safeAlgNoIOBufferSecurity) debugFlags |= FLAG_DEBUG_DISABLE_IOBUFFER_SECURING;

        // TARGET PROTECTED ALGORITHM
        algUUID.dataLen = algUUIDStr.GetLength();
        algUUID.pData = new BYTE[algUUID.dataLen];
        memset(algUUID.pData, 0, algUUID.dataLen);
        CCommonFnc::BYTE_ConvertFromHexStringToArray(algUUIDStr, algUUID.pData, &algUUID.dataLen);

        // INPUT DATA
        input.dataLen = m_inputData.GetLength();
        input.pData = new BYTE[input.dataLen];
        memset(input.pData, 0, input.dataLen);
        CCommonFnc::BYTE_ConvertFromHexStringToArray(inputData, input.pData, &input.dataLen);

        output.dataLen = APDU_MAX_OUTPUT_DATA_LEN;
        output.pData = new BYTE[output.dataLen];
        memset(output.pData, 0, output.dataLen);

        memset(&startTime, 0, sizeof(SYSTEMTIME));
        memset(&endTime, 0, sizeof(SYSTEMTIME));

        pLightApp->DisplayActualOperation(STR_ACTIVEOPER_PROCESSALG);

        GetSystemTime(&startTime); 
        if ((status = pLightApp->m_remoteManager.ProcessAlgData(&algUUID, &input, &output, debugFlags)) == STAT_OK) {
            GetSystemTime(&endTime); 
            message = "Process data success";

            if (m_outputDataHex) CCommonFnc::BYTE_ConvertFromArrayToHexString(output.pData, output.dataLen, &m_outputData);
            else CCommonFnc::BYTE_ConvertFromArrayToString(output.pData, output.dataLen, &m_outputData);

            // IF GAUSS EASTER ALGORITHM, DISPLAY MESSAGE
            if (m_safeAlg == 0) {
                CString easter;

                if ((output.pData[0] == 0x03) || (output.pData[0] == 0x04)) {
                    easter.Format(STR_EASTER_MESSAGE, year, output.pData[1]);
                    if (output.pData[0] == 0x03) easter += STR_MONTH_MARCH_4C;
                    else easter += STR_MONTH_APRIL_4C;

                    MessageBox(easter, STR_APP_NAME, MB_OK);
                }
            }


            UpdateData(FALSE);
        }
        else {
            GetSystemTime(&endTime); 
            message.Format("Process data fail with %s (%d)", ERROR_TO_STRING(status), status);
        }

        pLightApp->DisplayActualOperation();

        CCommonFnc::Time_GetTimeSpanSec(&startTime, &endTime, &floatTimeSpan);

        timeMessage.Format("--- elapsed timer (seconds): %f", floatTimeSpan);
        pLightApp->Log_InsertClearLine();
        pLightApp->Log_InsertString(timeMessage);
        pLightApp->Log_InsertString(message);

        delete[] algUUID.pData;
        delete[] input.pData;
        delete[] output.pData;
    }    

    UpdateData(FALSE);    	
}

BOOL CTabSafeAlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
    m_safeAlg = 0;
    UpdateData(FALSE);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CTabSafeAlg::OnStopAuth() 
{
    UpdateData(TRUE);

    int                 status = STAT_OK;
    CString             message;
    CString             timeMessage;
    SWA_PROFILE         swaProfile;
    DWORD               length = 0;
    BYTE                debugFlags = 0;
    SYSTEMTIME          startTime;
    SYSTEMTIME          endTime;
    float               floatTimeSpan;
     

    UpdateData(TRUE);

    if (m_seautNoIOBufferSecurity) debugFlags |= FLAG_DEBUG_DISABLE_IOBUFFER_SECURING;

    memset(&startTime, 0, sizeof(SYSTEMTIME));
    memset(&endTime, 0, sizeof(SYSTEMTIME));

    pLightApp->DisplayActualOperation(STR_ACTIVEOPER_SWAAUTH_STOP);

    // STOP AUTHENTIZATION BY SEAUT
    GetSystemTime(&startTime); 
    if ((status = pLightApp->m_remoteManager.StopAuth(debugFlags)) == STAT_OK) {
        GetSystemTime(&endTime); 
        
        message = "StopAuth success";
        
        pLightApp->SetAuthSWA(FALSE);
    }
    else {
        GetSystemTime(&endTime); 

        message.Format("StopAuth fail with %s (%d)", ERROR_TO_STRING(status), status);
    }

    pLightApp->DisplayActualOperation();
    
    CCommonFnc::Time_GetTimeSpanSec(&startTime, &endTime, &floatTimeSpan);

    timeMessage.Format("--- elapsed timer (seconds): %f", floatTimeSpan);
    pLightApp->Log_InsertClearLine();
    pLightApp->Log_InsertString(timeMessage);
    pLightApp->Log_InsertString(message);


    UpdateData(FALSE);
}

void CTabSafeAlg::OnChangeInputdata() 
{
}

void CTabSafeAlg::OnInputdataHexadecimal() 
{
    CString     temp;
    UpdateData(TRUE);

    if (m_inputDataHex) {
        // CONVERT FROM STRING TO HEX
        CCommonFnc::String_ConvertFromStringToHex(m_inputData, &temp);
        m_inputData = temp;
    }
    else {
        // CONVERT FROM HEX TO STRING
        CCommonFnc::String_ConvertFromHexToString(m_inputData, &temp);
        m_inputData = temp;
    }

    UpdateData(FALSE);
}

void CTabSafeAlg::OnOutputdataHexadecimal() 
{
    CString     temp;
    UpdateData(TRUE);

    if (m_outputDataHex) {
        // CONVERT FROM STRING TO HEX
        CCommonFnc::String_ConvertFromStringToHex(m_outputData, &temp);
        m_outputData = temp;
    }
    else {
        // CONVERT FROM HEX TO STRING
        CCommonFnc::String_ConvertFromHexToString(m_outputData, &temp);
        m_outputData = temp;
    }

    UpdateData(FALSE);
}
